<?php

namespace Modules\Qc\Http\Controllers;

use Exception;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Auth;
use Illuminate\Contracts\Support\Renderable;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Modules\Qc\Entities\DailyLabReportSb;
use Modules\Qc\Entities\DailyLabReportSbAttachment;

class DailyLabReportSbController extends Controller
{
    public function uploadAttachment(Request $request)
    {
        try {
            $token = $request->get('attachment_token');
            $file = $request->file('file');
            $file_name = $file->getClientOriginalName();
            $filename = uniqid() . '_' . time() . '.' . $file_name;
            $file->move(public_path('dlr_sb'), $filename);

            $dlr_sb_attachment = new DailyLabReportSbAttachment();
            $dlr_sb_attachment->dlr_sb_id = 0;
            $dlr_sb_attachment->path = $filename;
            $dlr_sb_attachment->token = $token;
            $dlr_sb_attachment->save();

            $responseBody = $this->responseBody(true, "DailyLabReportSbController", "uploaded", null);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "DailyLabReportSbController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }

    public function allDailyLabReportSbs()
    {
        try {
            $query = "SELECT
            qc_daily_lab_report_sb.*,
            qc_daily_lab_report_sb_attachments.path,
            hr_employee_sites.site
        FROM
            qc_daily_lab_report_sb_attachments
            INNER JOIN
            qc_daily_lab_report_sb
            ON
                qc_daily_lab_report_sb_attachments.dlr_sb_id = qc_daily_lab_report_sb.dlr_sb_id          
            INNER JOIN
            hr_employee_sites
            ON
                qc_daily_lab_report_sb.factory = hr_employee_sites.site_id
                GROUP BY
            qc_daily_lab_report_sb.dlr_sb_id";

            $records = DB::select($query);
            $result = array();
            foreach ($records as $record) {
                array_push($result, [
                    "dlr_sb_id" => $record->dlr_sb_id,
                    "reference_no" => $record->reference_no,
                    "date" => $record->date,
                    "factory" => $record->site,
                    "description" => $record->description,
                    "path" => $record->path,
                ]);
            }
            $responseBody = $this->responseBody(true, "DailyLabReportSbController", "allDailyLabReportSbs", $result);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "DailyLabReportSbController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }


    public function saveDailyLabReportSb(Request $request)
    {
        try {
            $sqr = new DailyLabReportSb();
            $sqr->user_id = Auth::user()->id;
            $sqr->reference_no = $request->get('reference_no');
            $sqr->date = $request->get('date');
            $sqr->factory = $request->get('factory');
            $sqr->description = $request->get('description');
            if ($sqr->save()) {
                $token = $request->get('attachment_token');
                $attachment = DailyLabReportSbAttachment::where('token', '=', $token)->first();
                if ($attachment) {
                    $attachment->dlr_sb_id = $sqr->dlr_sb_id;
                    $attachment->update();
                }
            }
            $responseBody = $this->responseBody(true, "DailyLabReportSbController", "saveDailyLabReportSb", $attachment);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "DailyLabReportSbController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }


    public function update(Request $request, $id)
    {
        try {
            $sqr = DailyLabReportSb::find($id);
            $sqr->user_id = Auth::user()->id;
            $sqr->reference_no = $request->get('reference_no');
            $sqr->date = $request->get('date');
            $sqr->factory = $request->get('factory');
            $sqr->description = $request->get('description');
            if ($sqr->update()) {
                $token = $request->get('attachment_token');
                $attachments = DailyLabReportSbAttachment::where('token', '=', $token)->get();
                if (count($attachments) > 1) {
                    if ($this->delete($id)) {
                        $attachment = DailyLabReportSbAttachment::where('token', '=', $token)->first();
                        $attachment->dlr_sb_id = (int) $id;
                        $attachment->update();
                    }
                }
                $responseBody = $this->responseBody(true, "DailyLabReportSbController", "updated", count($attachments));
            }
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "DailyLabReportSbController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }




    public function delete($id)
    {
        try {
            $attachment = DailyLabReportSbAttachment::where('dlr_sb_id', '=', $id)->get();
            foreach ($attachment as $data) {
                $data->delete();
            }
            return true;
        } catch (Exception $exception) {
            return false;
        }
    }


    public function deleteRecord($id)
    {
        try {
            $record = DailyLabReportSb::where('dlr_sb_id', '=', $id)->get();
            foreach ($record as $data) {
                $data->delete();
            }
            $attachment = DailyLabReportSbAttachment::where('dlr_sb_id', '=', $id)->get();
            foreach ($attachment as $data) {
                $data->delete();
            }
            $responseBody = $this->responseBody(true, "DailyLabReportSbController", "deleted", null);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "DailyLabReportSbController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }

    public function getDlrSbData($id)
    {
        try {
            $query = "SELECT
            qc_daily_lab_report_sb.*,
            qc_daily_lab_report_sb_attachments.token
        FROM
            qc_daily_lab_report_sb_attachments
            INNER JOIN
            qc_daily_lab_report_sb
            ON
                qc_daily_lab_report_sb_attachments.dlr_sb_id = qc_daily_lab_report_sb.dlr_sb_id
        WHERE
            qc_daily_lab_report_sb.dlr_sb_id = {$id}";

            $sqr = DB::select($query);

            $responseBody = $this->responseBody(true, "DailyLabReportSbController", "getDlrSbData", $sqr[0]);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "DailyLabReportSbcontroller", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }



    /**
     * responseBody
     * This is used to create response.
     * @param success This is the paramter require boolean
     * @param name This is the paramter require ui table name
     * @param message This is the paramter require message content
     * @param result This is the paramter require result as some of data to return client
     * @return Json This returns as response.
     */
    private function responseBody($success, $name, $message, $result)
    {
        $body = [
            "success" => $success,
            "message" => $message,
            "name" => $name,
            "result" => $result
        ];
        return $body;
    }
}