<?php

namespace Modules\Qc\Http\Controllers;

use Exception;
use Illuminate\Contracts\Support\Renderable;
use Illuminate\Support\Facades\DB;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Modules\Qc\Entities\DamageCheckingRecordsHeader;
use Modules\Qc\Entities\DamageCheckingRecordsData;
use Modules\Pp\Entities\CustomerOrderPlansData;
use Modules\Pp\Entities\MachineryPlans;

class DamageCheckingRecordsHeaderController extends Controller
{

    public function allRecords()
    {
        try {
            $query = "SELECT
            qc_damage_checking_records_headers.*,
            pp_customer_order_plans_data.manufacturing_order_no,        
            pp_customer_orders.purchase_order,
            st_products.product_name
        FROM
            qc_damage_checking_records_headers        
            LEFT JOIN
            pp_customer_order_plans_data
            ON
                qc_damage_checking_records_headers.customer_order_plan_data_id = pp_customer_order_plans_data.customer_order_plan_data_id
            LEFT JOIN
            pp_customer_orders
            ON
            qc_damage_checking_records_headers.customer_order_plan_data_id = pp_customer_orders.order_id
            LEFT JOIN 
			st_products 
			ON pp_customer_order_plans_data.product_id = st_products.product_primary_id
        GROUP BY
            qc_damage_checking_records_headers.dcr_id";

            $records = DB::select($query);
            $result = array();
            foreach ($records as $record) {
                $damageData = DamageCheckingRecordsData::where('dcr_id', $record->dcr_id)->get();
                $total_damage = 0;
                foreach ($damageData as $data) {
                    $total_damage += $data->seal_damage + $data->block_damage + $data->slab_damage + $data->print_damage + $data->packing_damage + $data->other_damage;
                }
                array_push($result, [
                    "dcr_id" => $record->dcr_id,
                    "purchase_order" => $record->purchase_order,
                    "product_name" => $record->product_name,
                    "manufacturing_order_no" => $record->manufacturing_order_no,
                    "trans_date_time" => $record->trans_date_time,
                    "total_damage" => $total_damage
                ]);
            }
            $responseBody = $this->responseBody(true, "DamageCheckingRecordsHeaderController", "allRecords", $result);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "DamageCheckingRecordsHeaderController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }

    public function getOrderNo()
    {
        try {
            $query = "SELECT 
            pp_customer_order_plans.customer_order_plan_id,
            pp_customer_orders.factory_po_num,
            st_products.product_name,
            pp_customer_order_plans_data.customer_order_plan_data_id
        FROM 
            pp_customer_order_plans
        LEFT JOIN 
            pp_customer_orders ON pp_customer_order_plans.order_id = pp_customer_orders.order_id
        LEFT JOIN 
            pp_customer_order_plans_data ON pp_customer_order_plans.customer_order_plan_id = pp_customer_order_plans_data.customer_order_plan_id
        LEFT JOIN 
            st_products ON pp_customer_order_plans_data.product_id = st_products.product_primary_id
		GROUP BY 
            pp_customer_order_plans.customer_order_plan_id 
        ORDER BY 
            pp_customer_order_plans.customer_order_plan_id DESC";
            $orders = DB::select($query);
            $result = array();

            foreach ($orders as $order) {
                array_push($result, [
                    "customer_order_plan_data_id" => $order->customer_order_plan_data_id,
                    "factory_po_num" => $order->factory_po_num,
                    "product_name" => $order->product_name,
                ]);
            }

            
            $responseBody = $this->responseBody(true, "DamageCheckingRecordsHeaderController", "getOrderNo", $result);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "DamageCheckingRecordsHeaderController", "error", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }

    public function getProductPrimaryId()
    {
        try {
            $query = "SELECT
            pp_machinery_plans.product_primary_id,
            st_products.product_name
        FROM
            pp_machinery_plans
            LEFT JOIN
            st_products
            ON
                pp_machinery_plans.product_primary_id = st_products.product_primary_id
        GROUP BY
            pp_machinery_plans.product_primary_id";
            $products = DB::select($query);
            $result = array();
            foreach ($products as $product) {
                array_push($result, [
                    "product_primary_id" => $product->product_primary_id,
                    "product_name" => $product->product_name,
                ]);
            }
            $responseBody = $this->responseBody(true, "DamageCheckingRecordsHeaderController", "getProductPrimaryId", $result);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "DamageCheckingRecordsHeaderController", "error", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }

    public function deleteRecord($id)
    {
        try {
            if ($this->delete($id)) {
                $rec_data = DamageCheckingRecordsHeader::where('dcr_id', '=', $id)->get();
                foreach ($rec_data as $data) {
                    $data->delete();
                }
                return true;
            }
        } catch (Exception $exception) {
            return false;
        }
    }

    public function delete($id)
    {
        try {
            $rec_data = DamageCheckingRecordsData::where('dcr_id', '=', $id)->get();
            foreach ($rec_data as $data) {
                $data->delete();
            }
            return true;
        } catch (Exception $exception) {
            return false;
        }
    }

    public function damageCheckingReport($id)
    {
        try {
            $query1 = "SELECT
            qc_damage_checking_records_headers.*,
            pp_customer_order_plans_data.manufacturing_order_no,
            st_products.product_name,
            pp_customer_orders.purchase_order
        FROM
            qc_damage_checking_records_headers        
            LEFT JOIN
            pp_customer_order_plans_data
            ON
                qc_damage_checking_records_headers.customer_order_plan_data_id = pp_customer_order_plans_data.customer_order_plan_data_id
            LEFT JOIN
            pp_customer_orders
            ON
                qc_damage_checking_records_headers.customer_order_plan_data_id = pp_customer_orders.order_id
            LEFT JOIN 
			st_products 
			ON pp_customer_order_plans_data.product_id = st_products.product_primary_id
        WHERE
	        qc_damage_checking_records_headers.dcr_id = {$id}";

            $records = DB::select($query1);

            $header = [
                "order_no" => "",
                "product" => "",
                "factory_order_number" => "",
                "trans_date_time" => "",
            ];


            if ($records) {
                $header["order_no"] = $records[0]->purchase_order;
                $header["product"] = $records[0]->product_name;
                $header["factory_order_number"] = $records[0]->manufacturing_order_no;
                $header["trans_date_time"] = $records[0]->trans_date_time;
            }

            $query2 = "SELECT
            qc_damage_checking_records_data.*,
            hr_att_shifts.shift_name
        FROM
            hr_att_shifts
            LEFT JOIN
            qc_damage_checking_records_data
            ON
                hr_att_shifts.shift_id = qc_damage_checking_records_data.shift_id
                WHERE
	        qc_damage_checking_records_data.dcr_id = {$id}";

            $data = DB::select($query2);

            $records[0]->request_data = $data;


            $responseBody = $this->responseBody(true, "DamageCheckingRecordsHeaderController", "damageCheckingReport", ["header" => $header, "body" => ["data" => $data]]);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "DamageCheckingRecordsHeaderController", "damageCheckingReport", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }




    /**
     * responseBody
     * This is used to create response.
     * @param success This is the paramter require boolean
     * @param name This is the paramter require ui table name
     * @param message This is the paramter require message content
     * @param result This is the paramter require result as some of data to return client
     * @return Json This returns as response.
     */
    private function responseBody($success, $name, $message, $result)
    {
        $body = [
            "success" => $success,
            "message" => $message,
            "name" => $name,
            "result" => $result
        ];
        return $body;
    }
}
