<?php

namespace Modules\Qc\Http\Controllers;

use Exception;
use Illuminate\Support\Facades\DB;
use Illuminate\Contracts\Support\Renderable;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Modules\Qc\Entities\PackingMaterialChecklist;
use Modules\Qc\Entities\PackingMaterialChecklistHeader;
use Modules\St\Entities\Suppliers;

class PackingMaterialChecklistHeaderController extends Controller
{
    public function allRecords()
    {
        try {
            $query = "SELECT
            qc_packing_material_checklist_headers.*,
            hr_employee_sites.site
        FROM
            hr_employee_sites
            INNER JOIN
            qc_packing_material_checklist_headers
            ON
                hr_employee_sites.site_id = qc_packing_material_checklist_headers.factory
        GROUP BY
            qc_packing_material_checklist_headers.packing_material_id";

            $records = DB::select($query);
            $result = array();
            foreach ($records as $record) {
                array_push($result, [
                    "packing_material_id" => $record->packing_material_id,
                    "reference_no" => $record->reference_no,
                    "factory" => $record->site,
                    "supplier_name" => $record->supplier_name,
                    "date" => $record->date,
                ]);
            }
            $responseBody = $this->responseBody(true, "PackingMaterialChecklistHeaderController", "allRecords", $result);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "PackingMaterialChecklistHeaderController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }


    public function deleteRecord($id)
    {
        try {
            if ($this->delete($id)) {
                $rec_data = PackingMaterialChecklistHeader::where('packing_material_id', '=', $id)->get();
                foreach ($rec_data as $data) {
                    $data->delete();
                }
                return true;
            }
        } catch (Exception $exception) {
            return false;
        }
    }

    public function delete($id)
    {
        try {
            $rec_data = PackingMaterialChecklist::where('packing_material_id', '=', $id)->get();
            foreach ($rec_data as $data) {
                $data->delete();
            }
            return true;
        } catch (Exception $exception) {
            return false;
        }
    }


    public function packingMaterialCheckListReport($id)
    {
        try {
            $query1 = "SELECT
            qc_packing_material_checklist_headers.*,
            hr_employee_sites.site 
        FROM
            hr_employee_sites
            INNER JOIN qc_packing_material_checklist_headers ON hr_employee_sites.site_id = qc_packing_material_checklist_headers.factory 
        WHERE
            qc_packing_material_checklist_headers.packing_material_id = {$id}";

            $records = DB::select($query1);

            $header = [
                "packing_material_id" => "",
                "reference_no" => "",
                "factory" => "",
                "supplier_name" => "",
                "strapping_rolls" => "",
                "clips" => "",
                "film_rolls" => "",
                "carton_pack" => "",
                "date" => "",
            ];


            if ($records) {
                $header["packing_material_id"] = $records[0]->packing_material_id;
                $header["reference_no"] = $records[0]->reference_no;
                $header["factory"] = $records[0]->site;
                $header["supplier_name"] = $records[0]->supplier_name;
                $header["strapping_rolls"] = $records[0]->strapping_rolls;
                $header["clips"] = $records[0]->clips;
                $header["film_rolls"] = $records[0]->film_rolls;
                $header["carton_pack"] = $records[0]->carton_pack;
                $header["date"] = $records[0]->date;
            }

            $query2 = "SELECT
            * 
        FROM
            qc_packing_material_checklists 
        WHERE
            packing_material_id = {$id}";

            $data = DB::select($query2);

            $records[0]->request_data = $data;


            $responseBody = $this->responseBody(true, "PackingMaterialChecklistHeaderController", "damageCheckingReport", ["header" => $header, "body" => ["data" => $data]]);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "PackingMaterialChecklistHeaderController", "damageCheckingReport", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }

    public function allSuppliers()
    {
        try {
            $products = Suppliers::select('supplier_id As id', 'supplier_name As value0')->get();
            return $this->responseBody(true, "PackingMaterialChecklistHeaderController", "allSuppliers", $products);
        } catch (Exception $exception) {
            return $this->responseBody(false, "PackingMaterialChecklistHeaderController", "error", $exception);
        }
    }



    /**
     * responseBody
     * This is used to create response.
     * @param success This is the paramter require boolean
     * @param name This is the paramter require ui table name
     * @param message This is the paramter require message content
     * @param result This is the paramter require result as some of data to return client
     * @return Json This returns as response.
     */
    private function responseBody($success, $name, $message, $result)
    {
        $body = [
            "success" => $success,
            "message" => $message,
            "name" => $name,
            "result" => $result
        ];
        return $body;
    }

}