<?php

namespace Modules\Qc\Http\Controllers;

use Exception;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Auth;
use Illuminate\Contracts\Support\Renderable;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Modules\Qc\Entities\ScaleCalibration;
use Modules\Qc\Entities\ScaleCalibrationAttachment;

class ScaleCalibrationController extends Controller
{
    public function uploadAttachment(Request $request)
    {
        try {
            $token = $request->get('attachment_token');
            $file = $request->file('file');
            $file_name = $file->getClientOriginalName();
            $filename = uniqid() . '_' . time() . '.' . $file_name;
            $file->move(public_path('scale_calibration'), $filename);

            $scale_calibration_attachment = new ScaleCalibrationAttachment();
            $scale_calibration_attachment->scale_calibration_id = 0;
            $scale_calibration_attachment->path = $filename;
            $scale_calibration_attachment->token = $token;
            $scale_calibration_attachment->save();

            $responseBody = $this->responseBody(true, "ScaleCalibrationController", "uploaded", null);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "ScaleCalibrationController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }

    public function allRecords()
    {
        try {
            $query = "SELECT
            qc_scale_calibrations.*,
            qc_scale_calibration_attachments.path,
            hr_employee_sites.site
        FROM
            qc_scale_calibration_attachments
            INNER JOIN
            qc_scale_calibrations
            ON
                qc_scale_calibration_attachments.scale_calibration_id = qc_scale_calibrations.scale_calibration_id        
            INNER JOIN
            hr_employee_sites
            ON
                qc_scale_calibrations.factory = hr_employee_sites.site_id
                GROUP BY
            qc_scale_calibrations.scale_calibration_id";

            $records = DB::select($query);
            $result = array();
            foreach ($records as $record) {
                array_push($result, [
                    "scale_calibration_id" => $record->scale_calibration_id,
                    "reference_no" => $record->reference_no,
                    "date" => $record->date,
                    "factory" => $record->site,
                    "description" => $record->description,
                    "path" => $record->path,
                ]);
            }
            $responseBody = $this->responseBody(true, "ScaleCalibrationController", "allRecords", $result);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "ScaleCalibrationController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }


    public function saveRecord(Request $request)
    {
        try {
            $scale = new ScaleCalibration();
            $scale->user_id = Auth::user()->id;
            $scale->reference_no = $request->get('reference_no');
            $scale->date = $request->get('date');
            $scale->factory = $request->get('factory');
            $scale->description = $request->get('description');
            if ($scale->save()) {
                $token = $request->get('attachment_token');
                $attachment = ScaleCalibrationAttachment::where('token', '=', $token)->first();
                if ($attachment) {
                    $attachment->scale_calibration_id = $scale->scale_calibration_id;
                    $attachment->update();
                }
            }
            $responseBody = $this->responseBody(true, "ScaleCalibrationController", "saveRecord", $attachment);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "ScaleCalibrationController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }


    public function update(Request $request, $id)
    {
        try {
            $scale = ScaleCalibration::find($id);
            $scale->user_id = Auth::user()->id;
            $scale->reference_no = $request->get('reference_no');
            $scale->date = $request->get('date');
            $scale->factory = $request->get('factory');
            $scale->description = $request->get('description');
            if ($scale->update()) {
                $token = $request->get('attachment_token');
                $attachments = ScaleCalibrationAttachment::where('token', '=', $token)->get();
                if (count($attachments) > 1) {
                    if ($this->delete($id)) {
                        $attachment = ScaleCalibrationAttachment::where('token', '=', $token)->first();
                        $attachment->scale_calibration_id = (int)$id;
                        $attachment->update();
                    }
                }
                $responseBody = $this->responseBody(true, "ScaleCalibrationController", "updated", count($attachments));
            }
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "ScaleCalibrationController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }




    public function delete($id)
    {
        try {
            $attachment = ScaleCalibrationAttachment::where('scale_calibration_id', '=', $id)->get();
            foreach ($attachment as $data) {
                $data->delete();
            }
            return true;
        } catch (Exception $exception) {
            return false;
        }
    }


    public function deleteRecord($id)
    {
        try {
            $record = ScaleCalibration::where('scale_calibration_id', '=', $id)->get();
            foreach ($record as $data) {
                $data->delete();
            }
            $attachment = ScaleCalibrationAttachment::where('scale_calibration_id', '=', $id)->get();
            foreach ($attachment as $data) {
                $data->delete();
            }
            $responseBody = $this->responseBody(true, "ScaleCalibrationController", "deleted", null);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "ScaleCalibrationController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }

    public function getData($id)
    {
        try {
            $query = "SELECT
            qc_scale_calibrations.*,
            qc_scale_calibration_attachments.token
        FROM
            qc_scale_calibration_attachments
            INNER JOIN
            qc_scale_calibrations
            ON
                qc_scale_calibration_attachments.scale_calibration_id = qc_scale_calibrations.scale_calibration_id
        WHERE
            qc_scale_calibrations.scale_calibration_id = {$id}";

            $scale =  DB::select($query);

            $responseBody = $this->responseBody(true, "ScaleCalibrationController", "getData", $scale[0]);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "ScaleCalibrationController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }



    /**
     * responseBody
     * This is used to create response.
     * @param success This is the paramter require boolean
     * @param name This is the paramter require ui table name
     * @param message This is the paramter require message content
     * @param result This is the paramter require result as some of data to return client
     * @return Json This returns as response.
     */
    private function responseBody($success, $name, $message, $result)
    {
        $body = [
            "success" => $success,
            "message" => $message,
            "name" => $name,
            "result" => $result
        ];
        return $body;
    }
}
